; #########################################################################

  ; --------------------------------------------------
  ; This procedure was written by Raymond Filiatreault
  ; --------------------------------------------------

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap :none  ; case sensitive

      include \masm32\include\Mix.inc

      includelib \masm32\lib\Mix.lib

    .code

; #########################################################################

MixArctan proc public xInt1:DWORD, xInt2:DWORD
        
    ; -------------------------------------------------------------
    ; Computes the arctan of a fractional value per the following
    ; Taylor series:
    ;
    ;           arctan(x) = x - x^3/3 + x^5/5 ....
    ;
    ; eax, ecx, edx used
    ;
    ; xInt1:    Actual value of the tangent in a Mixed Integer format or
    ;           1st parameter to calculate the tangent
    ;
    ; xInt2:    2nd parameter to calculate the tangent or must be 0 if
    ;           the MInt1 parameter is the actual value of the tangent
    ;
    ; EXAMPLE:
    ;
    ;       invoke MixArctan, var1, var2
    ;
    ; The result is returned in EAX in a Mixed Integer format. If either
    ; parameter is the error code 80000000h, it is returned in EAX.
    ;
    ; The provided parameters can be positive or negative. Both parameters
    ; can also be regular integers.
    ;
    ; If the second parameter is 0, the first parameter will be treated as
    ; the actual value of the tangent and must be in the Mixed Integer
    ; format. In this case, if a returned value in the 90-270 degree range
    ; is desired, use -10000h as the second parameter instead of 0.
    ;
    ; The value of the returned angle will depend on the signs of the two
    ; parameters:
    ;   xInt1(+)    xInt2(+)    0-90    degrees
    ;   xInt1(+)    xInt2(-)    90-180  degrees
    ;   xInt1(-)    xInt2(-)    180-270 degrees
    ;   xInt1(-)    xInt2(+)    270-360 degrees
    ;
    ; Note: In order to keep computation time within reason, a straight
    ; line interpolation is used if the tan value is .95 to 1.05. The
    ; resulting angle is still within 1/100th degree of accuracy compared
    ; to 1/200th degree of accuracy by regular computation in the .95-1.0
    ; range. (As the tan value approached the .95, the time required on
    ; a P3-550e was 200 nanoseconds.)
    ;
    ; -------------------------------------------------------------

    LOCAL   sums        :DWORD
    LOCAL   tanvalue    :DWORD
    LOCAL   posMInt1    :DWORD
    LOCAL   posMInt2    :DWORD

    atan95  EQU 2B87FDh
    tan95   EQU 0F333h
    
    mov eax,xInt1
    .if eax == ERROR
        ret
    .endif

    mov ecx,xInt2
    .if ecx == ERROR
        mov eax,ecx
        ret
    .endif

    push ebx
    push esi

    or  eax,eax
    jns @F
    neg eax

  @@:
    mov posMInt1,eax

    or  ecx,ecx
    jnz @F
    .if eax > 10000h
        mov posMInt2,10000h
        invoke MixDiv,10000h,eax  ;get reciprocal if tan value was > 1
        jmp tangent
    .endif

  @@:
    jns @F
    neg ecx

  @@:
    mov posMInt2,ecx
    .if ecx < eax
        xchg ecx,eax
    .endif
    invoke MixDiv,eax,ecx

tangent:
    mov tanvalue,eax
    mov sums,eax

    .if eax >= tan95    ;use a straight line interpolation
                        ;if the tan value is .95 or higher
        sub eax,tan95
        mov edx,2D0000h
        sub edx,atan95
        mul edx
        mov ecx,10000h
        sub ecx,tan95
        div ecx
        add eax,atan95
        jmp quadrant
    .endif
    
    mov esi,1           ;for current number starting at 1
    xor ebx,ebx         ;for sign flag starting at +

  .while eax != 0

      xor ebx,80000000h   ;change sign flag
      
      inc esi
      inc esi
     
      mov eax,tanvalue
      mov ecx,esi
    @@:
      dec ecx
      jz  @F
      push ecx
      invoke MixMul,eax,tanvalue     ;(tan)^esi
      pop ecx
      jmp @B
  
    @@:
      xor edx,edx
      div esi
      or  ebx,ebx
      jns @F
      neg eax
    @@:
      add sums,eax

  .endw

    mov eax,sums
    mov ecx,180 shl 16
    imul ecx
    mov ecx,MixPI
    idiv ecx                 ;angle now in degrees 0-45

quadrant:
    mov ecx,posMInt1
    cmp ecx,posMInt2
    jb  @F
    mov ecx,90 shl 16
    sub eax,ecx
    neg eax                 ;angle now 0-90 degrees

  @@:
    mov ecx,xInt1
    mov edx,xInt2
    or  ecx,ecx
    js  @F
    or  edx,edx
    jns eXit                ;1st quadrant no change

    mov ecx,180 shl 16
    sub eax,ecx
    neg eax                 ;2nd quadrant 90-180
    jmp eXit

  @@:
    or  edx,edx
    js  @F
    mov ecx,360 shl 16
    neg eax
    add eax,ecx             ;4th quadrant 270-360
    jmp eXit

  @@:
    mov ecx,180 shl 16
    add eax,ecx             ;3rd quadrant 180-270
    
eXit:
    pop esi
    pop ebx    
    ret
    
MixArctan endp

; #########################################################################

end

